/*
  Project: Water flow meter YF- S201, 5V relay module, 12V solenoid valve, LCD1602 I2C module
  Function: Measure of Water passed through the solenoid valve, close the solenoid valve when result is 15 ltrs

  Connect the red wire to +5V,
  the black wire to common ground
  and the yellow sensor wire to pin #2

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada  for Adafruit Industries.
  BSD license, check license.txt for more information
  All text above must be included in any redistribution*/
//*********************************************************
#include <LiquidCrystal_I2C.h> //include LiquidCrystal Library
//*********************************************************
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);//initialize the library with the numbers of the interface pins
#define FLOWSENSORPIN 2 //water flow sensor attached to Arduino digital pin 2
#define relayPin 4     //5v relay module attached to Arduino digital pin 4
volatile uint16_t pulses = 0; // count how many pulses
volatile uint8_t lastflowpinstate; // track the state of the pulse pin
volatile uint32_t lastflowratetimer = 0; // you can try to keep time of how long it is between pulses
volatile float flowrate; // and use that to calculate a flow rate
// Interrupt is called once a millisecond, looks for any pulses from the sensor!
SIGNAL(TIMER0_COMPA_vect) {
  uint8_t x = digitalRead(FLOWSENSORPIN);

  if (x == lastflowpinstate) {
    lastflowratetimer++;
    return; // nothing changed!
  }

  if (x == HIGH) {
    //low to high transition!
    pulses++;
  }
  lastflowpinstate = x;
  flowrate = 1000.0;
  flowrate /= lastflowratetimer;  // in hertz
  lastflowratetimer = 0;
}

void useInterrupt(boolean v) {
  if (v) {
    // Timer0 is already used for millis() - we'll just interrupt somewhere
    // in the middle and call the "Compare A" function above
    OCR0A = 0xAF;
    TIMSK0 |= _BV(OCIE0A);
  } else {
    // do not call the interrupt function COMPA anymore
    TIMSK0 &= ~_BV(OCIE0A);
  }
}

void setup() {
  Serial.begin(9600);
  Serial.print("---Water flow sensor---");
  lcd.begin(16, 2);                            //16X2 lcd display
  lcd.setBacklightPin(3, POSITIVE);           //switch on the backlight
  lcd.setBacklight(HIGH);
  lcd.setCursor(0, 0);                       //setting display position
  lcd.print("Aqua counter");
  pinMode(FLOWSENSORPIN, INPUT); //sets the FLOWSENSORPIN as an INPUT
  pinMode(relayPin, OUTPUT);//sets the relayPin as OUTPUT
  digitalWrite(relayPin, HIGH);
  digitalWrite(FLOWSENSORPIN, HIGH);//optional Internal Pull-Up
  lastflowpinstate = digitalRead(FLOWSENSORPIN);
  useInterrupt(true);
  delay(2000);
  lcd.clear();
}

void loop()
{
  lcd.setCursor(0, 0);
  lcd.print("Pulses:"); lcd.print(pulses, DEC);
  lcd.print(" Hz:");
  lcd.print(flowrate);
  //lcd.print(flowrate);
  Serial.print("Freq: "); Serial.println(flowrate);
  Serial.print("Pulses: "); Serial.println(pulses, DEC);

  // if a plastic sensor use the following calculation
  // Sensor Frequency (Hz) = 7.5 * Q (Liters/min)
  // Liters = Q * time elapsed (seconds) / 60 (seconds/minute)
  // Liters = (Frequency (Pulses/second) / 7.5) * time elapsed (seconds) / 60
  // Liters = Pulses / (7.5 * 60)
  float liters = pulses;
  liters /= 7.5;
  liters /= 60.0;

  /*
    // if a brass sensor use the following calculation
    float liters = pulses;
    liters /= 8.1;
    liters -= 6;
    liters /= 60.0;
  */
  Serial.print(liters); Serial.println(" Liters");
  lcd.setCursor(0, 1);
  lcd.print(liters); lcd.print(" Litres        ");
  if (liters >= 15) //water limit set
  {
    digitalWrite(relayPin, LOW);
  }
  else {
    digitalWrite(relayPin, HIGH);
  }
  delay(2000);
  lcd.clear();


}
